#!/bin/bash

echo "change_governor - show or set governor (set requires root privileges)"
echo "usage: [sudo] change_governor [o|ondemand|f|performance|p|powersave|u|userspace|i|interactive|c|conservative]"
echo "filemode: change_governor file filename.txt"
echo " requires strict format: "
echo "     governor=0,1,2,3,4,5"
echo "     a7freq=NUMBER in KHz [used for userspace] otherwise used as minfreq"
echo "     a15freq=NUMBER in KHz [used for userspace] otherwise used as minfreq"
echo "     a7maxfreq=NUMBER in KHz [used as maxfreq if given]"
echo "     a15maxfreq=NUMBER in KHz [used as maxfreq if given]"
echo "governor in use:"

getgov=$(cat /sys/devices/system/cpu/cpu*/cpufreq/scaling_governor)

a7freq=
a15freq=
a7maxfreq=
a15maxfreq=

case $1 in
      o|ondemand)     governor=ondemand;;
      f|performance)  governor=performance;;
      p|powersave)    governor=powersave;;
      u|userspace)    governor=userspace;;
      i|interactive)  governor=interactive;;
      c|conservative) governor=conservative;;
      file) 
          line1="$(sed -n 1p $2 | awk -F = '{ print $2 }')"
          a7freq="$(sed -n 2p $2 | awk -F = '{ print $2 }')"
          a15freq="$(sed -n 3p $2 | awk -F = '{ print $2 }')"
          a7maxfreq="$(sed -n 4p $2 | awk -F = '{ print $2 }')"
          a15maxfreq="$(sed -n 5p $2 | awk -F = '{ print $2 }')"
          #echo $line1
          #echo $a7freq
          #echo $a15freq
          case $line1 in
            0) governor=ondemand;;
            1) governor=performance;;
            2) governor=powersave;;
            3) governor=userspace;;
            4) governor=interactive;;
            5) governor=conservative;;
          esac
          
        ;;
      *) echo $getgov
      exit;;
esac

if [[ "$1" != "file" ]]; then
case $2 in
      a7) echo $governor > /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor;;
      a15) echo $governor > /sys/devices/system/cpu/cpu4/cpufreq/scaling_governor;;
      *) echo $governor > /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor
         echo $governor > /sys/devices/system/cpu/cpu4/cpufreq/scaling_governor;;
esac
else

    echo performance > /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor
    echo $governor > /sys/devices/system/cpu/cpu4/cpufreq/scaling_governor

    #set frequencies if necessary
    if [[ "$governor" == "userspace" ]] ; then
    
        #echo $a7freq > /sys/devices/system/cpu/cpu0/cpufreq/scaling_setspeed
        echo $a15freq > /sys/devices/system/cpu/cpu4/cpufreq/scaling_setspeed
    
    elif [[ "$a15maxfreq" != "" ]]; then
    	echo $a15freq > /sys/devices/system/cpu/cpu4/cpufreq/scaling_min_freq
    	echo $a15maxfreq > /sys/devices/system/cpu/cpu4/cpufreq/scaling_max_freq
    fi

    if [[ "$a7freq" != "" ]]; then
    	echo $a7freq > /sys/devices/system/cpu/cpu0/cpufreq/scaling_min_freq
    fi
    if [[ "$a7maxfreq" != "" ]]; then
    	echo $a7freq > /sys/devices/system/cpu/cpu0/cpufreq/scaling_max_freq
    fi


fi

# A7
#echo $governor > /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor

# A15
#echo $governor > /sys/devices/system/cpu/cpu4/cpufreq/scaling_governor


#for g in /sys/devices/system/cpu/cpu*/cpufreq/scaling_governor; do echo $governor > $g; done

echo $(cat /sys/devices/system/cpu/cpu*/cpufreq/scaling_governor)
